function nan_andrewsplot(X,groups,quant,stdize)
// Andrews plot for multivariate data.
// Calling Sequence
// nan_andrewsplot(X)
// nan_andrewsplot(X,groups)
// nan_andrewsplot(X,groups,quant)
// nan_andrewsplot(X,groups,quant,stdize)
// Parameters
// X :  Rows of X correspond to observations, columns to variables.
// groups: plots the data in different groups   with different colors, must be a numeric array or character matrix,
// quant: if it is not empty it plots only the median and the    quant and (1-quant) quantiles of f(t) at each value of t.  This is   useful if X contains many observations.
// stdize = 'off' : do nothing
// stdize ='on' : normalize each column of X (mean = 0, deviation = 1)
// stdize = 'pca' : instead of X   the principal component scores of X is used, in order of decreasing eigenvalue.
// stdize = 'pcastd' : uses the standardized   principal component scores.
// Description
//   
//  An  Andrews plot is a graphical representation of the observed values ​​of several metric scaled variables X_j (j = 1, ..., p) in a two-dimensional coordinate system in which each multivariate data point is represented by a function.
//  
//  f(t) is defined for the j-th observation as
//  
//  f(t) = X(j,1)/sqrt(2) + X(j,2)*sin(2*pi*t) + X(j,3)*cos(2*pi*t) + ...
//  
//  Elements with similar values ​​will have similar curves.  This is possible because the Andrews plot preserves the distances of data points and the mean values ​​of the variables. The Andrews plot is suitable for the detection of outliers and for finding clusters.
//  
//
//   nan_andrewsplot treats NaNs in X as missing values, and remove the corresponding rows. 
//   Examples
//
//      // make a grouped plot of the raw data
//       loadmatfile(nan_getpath()+"/demos/data/iris.mat");
//      nan_andrewsplot(meas,species);
//
//      // plot only the median and quartiles of each group
//      nan_andrewsplot(meas,species,.25);
//      
//   Bibliography
//    Andrews, D. F. (1972), Plots of high-dimensional data, Biometrics 28, S.125-136 
// See also
// nan_parallelcoords


[nargout,nargin]=argn(0);
if nargin < 1
    error('At least one input Argument is needed!');
end
[n,p] = size(X);

if nargin<2 then
   groups=[];
   quant=[];
   stdize='off';
elseif nargin<3 then
      quant=[];
   stdize='off';
elseif nargin<4 then
   stdize='off';
end;


if ~isempty(quant)
    if ~or(type(quant)==[1 5 8]) | ~sum(length(quant))==1 | ~(0 < quant & quant < 1)
        error('quant must be a scalar between 0 and 1.');
    end
    quant = min(quant,1-quant);
end


if isempty(groups)
    gidx = ones(n,1);
    ngroups = 1;
    gnames = [];

else
    [gidx,gnames] = nan_grp2idx(groups);
    ngroups = size(gnames,'*');
    if max(size(gidx)) ~= n
        error(' the number or rows of groups and X must be the same!');
    end
end
obsinds = 1:n;
nans = find(isnan(gidx) | or(isnan(X),2));
if ~isempty(nans)
    X(nans,:) = [];
    gidx(nans) = [];
    obsinds(nans) = [];
    n = size(X,1);
end

if type(stdize)==10
    select convstr(stdize)
    case 'off' then
        
    case 'on' then
        if n > 1
            X = (X - repmat(nan_mean(X,1),n,1))./ repmat(nan_std(X,[],1),n,1);
        elseif n == 1
            X = zeros(X);
        end
        
    case 'pca' then 
        if ~isempty(X)
            [dum,X,variance] = princomp(X); 
            
        end
      case 'pcastd' then 
        if ~isempty(X)
            [dum,X,variance] = princomp(X);
                tol = max(n,p)*%eps;
                variance(variance<tol) = 1;
                X = X ./ repmat(sqrt(variance(:)'),n,1);
        end
        
    else
        error(' stdize must be either ''off'', ''on'', ''pca'', or ''pcastd''.');
    end
else
    error('stdize must be a string ');
end

t = 0:.001:1;
omega = 2*%pi*(1:floor(p/2));
omegaSin = omega(1:floor(p/2));
omegaCos = omega(1:floor((p-1)/2));
A = [ones(1,length(t))./sqrt(2); sin(omegaSin' * t); cos(omegaCos' * t)];
F = X(:,[1 (2:2:p) (3:2:p)])*A;


scf();
cax=gca();

colorlist=["b","r","g","k","y","c","m"];
ncolors = size(colorlist,'*');

hh = list();
lgndh = [];;
for grp = 1:ngroups
    colorNr = (pmodulo(grp-1,ncolors)+1);
    mbrs = find(gidx == grp);
   
    if isempty(mbrs)
        plot(t,%nan*ones(size(t,1),size(t,2)), 'LineStyle','-','Color',colorlist(colorNr));
        
    elseif isempty(quant)
        plot(t,F(mbrs,:)', 'LineStyle','-', 'Color',colorlist(colorNr));
        hh(grp) = gce();
    else
        if length(mbrs) == 1
            per = [.5 .5 .5];
            q = repmat(F(mbrs,:),3,1);
        else
            per = [.50 quant 1-quant];
            q = prctile(F(mbrs,:), 100*per);
        end
        plot(t,q(1,:), 'LineStyle','-', 'Color',colorlist(colorNr));
        hh(grp) = gce();
        plot(t,q(2:3,:), 'LineStyle',':', 'Color',colorlist(colorNr));
        //hh((grp-1)*2 + (2)) = gce();
       
        
    end
    
     if ~isempty(groups) & ~isempty(mbrs)
         if isempty(lgndh)
             lgndh=hh(grp).children(1);
         else
              lgndh= [lgndh; hh(grp).children(1)];
         end;
     end
end



xlabel('t'); ylabel('f(t)');

if ~isempty(groups)
    tt = 1:size(lgndh,'*');  
    legend(lgndh,gnames(tt,:));
end

endfunction


