function [pval, f, df_b, df_w] = nan_anova (y, g,alpha)
// Perform a one-way analysis of variance (ANOVA)
// Calling Sequence
// [pval, f, df_b, df_w] = anova (y)
// [pval, f, df_b, df_w] = anova (y,alpha)
// [pval, f, df_b, df_w] = anova (y,g)
// Parameters
// 
// Description
// Perform a one-way analysis of variance (ANOVA).  The goal is to test
// whether the population means of data taken from k different
// groups are all equal.
//
// Data may be given in a single vector y with groups specified by
// a corresponding vector of group labels g (e.g., numbers from 1
// to k).  This is the general form which does not impose any
// restriction on the number of data in each group or the group labels.
//
// If y is a matrix and g is omitted, each column of y
// is treated as a group.  This form is only appropriate for balanced
// ANOVA in which the numbers of samples from each group are all equal.
//
// Under the null of constant means, the statistic f follows an F
// distribution with df_b and df_w degrees of freedom.
//
// The p-value (1 minus the CDF of this distribution at @var{f}) is
// returned in @var{pval}.
//
// If no output argument is given, the standard one-way ANOVA table is
// printed.
// Examples
// y=[17,25,22,26;19,27,21,24;20,18,19,30;24,22,26,28];
// nan_anova(y,0.01);
// 
// One-way ANOVA Table:
// 
// Source of Variation   Sum of Squares    df  Empirical Var
// *********************************************************
// Between Groups              104.0000     3        34.6667
// Within Groups               118.0000    12         9.8333
// ---------------------------------------------------------
// Total                       222.0000    15
// 
// Test Statistic f              3.5254
// p-value                       0.0487
// Fcrit (alpha=0.0100)          5.9525
// 
// y_list=list(y(:,1),y(:,2),y(:,3),y(:,4));
// nan_anova(y_list,0.01);
// 
// y_vec=y(:);
// groups=[1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4];
// nan_anova(y_vec,groups,0.01);
//
// Authors
// Copyright (C) 1995-2011 Kurt Hornik
// H. Nahrstaedt - 2012


//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.
		
// Author: KH <Kurt.Hornik@wu-wien.ac.at>
// Description: One-way analysis of variance (ANOVA)

[nargout,nargin]=argn(0);

  if ((nargin < 1) | (nargin > 3))
    error("wrong number of parameters") 
  elseif (nargin < 3) & type(y)==[15] then
    if (length (y)==1)
      error ("anova: for anova (Y), Y must not be a vector");
    end
    if nargin==2 then
      alpha=g;
    else
      alpha=[];
    end;
    k=length(y);
    group_count=zeros(1,k);
    group_mean=zeros(1,k);
    n=0;
    y_tmp=y;
    y=[];
    for i=1:k
      group_count(i)=length(y_tmp(i));
      group_mean(i) = nan_mean (y_tmp(i));
      n=n+group_count(i);
      tmp=y_tmp(i);
      y=[y;tmp(:)];
    end
  elseif (~isvector (y)) & nargin<3 & or(type(y)==[1 5 8]) then
    if (isvector (y))
      error ("anova: for anova (Y), Y must not be a vector");
    end
    if nargin==2 then
      alpha=g;
    else
      alpha=[];
    end;
    [group_count, k] = size (y);
    n = group_count * k;
    group_mean = mean (y,'r');
  else
    if nargin<3 then
      alpha=[];
    end;
    if (~ isvector (y))
      error ("anova: for anova (Y, G), Y must be a vector");
    end
    n = length (y);
    if (~ isvector (g) | (length (g) ~= n))
      error ("anova: G must be a vector of the same length as Y");
    end
    s = mtlb_sort (g);
    i = find (s (2 : n) > s(1 : (n-1)));
    k = length (i) + 1;
    if (k == 1)
      error ("anova: there should be at least 2 groups");
    else
      group_label = s ([1, (matrix (i, 1, k-1) + 1)]);
    end
    for i = 1 : k;
      v = y (find (g == group_label (i)));
      group_count (i) = length (v);
      group_mean (i) = mean (v);
    end

  end



  total_mean = mean (y);
  SSB = sum (group_count .* (group_mean - total_mean) .^ 2);
  SST = nan_sumsq (matrix (y, n, 1) - total_mean);
  SSW = SST - SSB;
  df_b = k - 1;
  df_w = n - k;
  v_b = SSB / df_b;
  v_w = SSW / df_w;
  f = v_b / v_w;
  pval = 1 - distfun_fcdf (f, df_b, df_w);
  
  //if (length(df_b)==1 & length(df_w)==1)
      //pval = 1- (1 - nan_betainc (1 ./ (1 + df_b .* f ./ df_w), df_w / 2, df_b / 2));


      
 if ~isempty(alpha)
  fcrit=cdff("F",df_b,df_w,1-alpha,alpha)
 end;

  //if (nargout == 0)
    // This eventually needs to be done more cleanly ...
    printf ("\n");
    printf ("One-way ANOVA Table:\n");
    printf ("\n");
    printf ("Source of Variation   Sum of Squares    df  Empirical Var\n");
    printf ("*********************************************************\n");
    printf ("Between Groups       %15.4f  %4d  %13.4f\n", SSB, df_b, v_b);
    printf ("Within Groups        %15.4f  %4d  %13.4f\n", SSW, df_w, v_w);
    printf ("---------------------------------------------------------\n");
    printf ("Total                %15.4f  %4d\n", SST, n - 1);
    printf ("\n");
    printf ("Test Statistic f     %15.4f\n", f);
    printf ("p-value              %15.4f\n", pval);
    if ~isempty(alpha) then
    printf ("Fcrit (alpha=%1.4f) %15.4f\n",alpha,fcrit);
    end;
    printf ("\n");
  //end

endfunction
