function [pval, chisq, df] = nan_barttest (varargin)
//Bartlett test for homogeneity of variances
//Calling Sequence
//[pval, chisq, df] = nan_barttest (x1,x2,...,xn)
//Parameters
// pval: p-value
// Description
// Perform a Bartlett test for the homogeneity of variances in the data
// vectors x1, x2, ..., xk, where k > 1.
//
// Under the null of equal variances, the test statistic chisq
// approximately follows a chi-square distribution with df degrees of
// freedom.
//
// The p-value (1 minus the CDF of this distribution at @var{chisq}) is
// returned in pval.
//
// Authors
// Kurt Hornik Kurt.Hornik@wu-wien.ac.at
// H. Nahrstaedt - 2012

// Copyright (C) 1995-2011 Kurt Hornik
//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.
[nargout,nargin]=argn(0);

  k = nargin;
  if (k < 2)
   error("wrong number of parameters") ;
  end

  f = zeros (k, 1);
  v = zeros (k, 1);

  for i = 1 : k;
    x = varargin(i);
    if (~ isvector (x))
      error ("bartlett_test: all arguments must be vectors");
    end
    f(i) = length (x) - 1;
    v(i) = nan_var (x);
  end

  f_tot = sum (f);
  v_tot = sum (f .* v) / f_tot;
  c     = 1 + (sum (1 ./ f) - 1 / f_tot) / (3 * (k - 1));
  chisq = (f_tot * log (v_tot) - sum (f .* log (v))) / c;
  df    = k;
  //pval  = 1 - chi2cdf (chisq, df);
  //pval  = 1 - gamcdf (chisq, df / 2, 2);
  //pval =1 -  gammainc (chisq ./ 2, df / 2);
  pval =1 -  cdfgam("PQ",chisq ./ 2,df / 2,ones(df));

  if (nargout == 0)
    printf("  pval: %g\n", pval);
  end

endfunction
