function T = nan_cluster(Z,c)
// constructs T from the agglomerative hierarchical cluster tree, Z, as generated by the nan_linkage function
// Calling Sequence
// T = cluster(Z,c)
// Parameters
// Z : matrix of size (m – 1)-by-3, which is generated by the nan_linkage function
// c: threshold for cutting Z into T
// T: T is a vector of size m containing the cluster assignments of each observation.
// Description
//T = cluster(Z,c) constructs T from the agglomerative hierarchical
// cluster tree, Z, as generated by the nan_linkage function. Z is a matrix of size (m – 1)-by-3,
// where m is the number of observations in the original data. c is a threshold
// for cutting Z into T. T are formed when a node and all of its
// subnodes have inconsistent value less than c. All leaves at or below the node 
// are grouped into a cluster. T is a vector of size m containing the cluster
// assignments of each observation.
// Examples
// //Compare clusters from Fisher iris data with species:
//   loadmatfile(nan_getpath()+"/demos/data/iris.mat");
//  d = nan_pdist(meas);
//  Z = nan_linkage(d);
//  c = nan_cluster(Z,145);
//  nan_crosstab(c,species)
//
// Authors
//  Stephane Franiatte, B.Eng. stephane.franiatte@gmail.com - 09/03/2013

    class_att = 0;
    T = zeros(size(Z,1)+1,1);
    d = size(Z,1)+2:2*size(Z,1)+1;
    d = d';
    Z = [Z d];
    
    for i=1:size(Z,1)
        if(i <= c)
            if(Z(i,1) > size(Z,1)+1 & Z(i,2) < size(Z,1)+1)
                Z(i,5) = Z(Z(:,4) == Z(i,1),5);
            elseif(Z(i,2) > size(Z,1)+1 & Z(i,1) < size(Z,1)+1)
                Z(i,5) = Z(Z(:,4) == Z(i,2),5);
            elseif(Z(i,1) > size(Z,1)+1 & Z(i,2) > size(Z,1)+1)
                // choose one branch, assign other's class to it
                Z = classeArbreRec(Z(i,1),Z(Z(:,4) == Z(i,2),5),Z);
                // assign class to current line
                Z(i,5) = Z(Z(:,4) == Z(i,2),5);
            else
                class_att = class_att + 1;
                Z(i,5) = class_att;
            end
        else
            class_att = class_att + 1;
            Z(i,5) = class_att;
        end
    end
    
    for i=1:size(T,1)
        if(max(bool2s(Z(:,1) == i)) > 0)
            T(i,1) = Z(Z(:,1) == i,5);
        else
            T(i,1) = Z(Z(:,2) == i,5);
        end
    end
    
    class_att = 0;
    
    for i=1:size(T,1)
        if(T(i,1) > class_att)
            class_att = class_att + 1;
            T(T(:,1) == T(i,1),1) = class_att;
        end
    end

endfunction

// Recursive function to allocate label to every node until it reaches a leaf
function Z = classeArbreRec(noeud,classe,Z)

    if(noeud > size(Z,1)+1)
        // intermediate nodes need to change their classes too, or we may have
        // reach a leaf
        Z(Z(:,4) == noeud,5) = classe;
        if(Z(Z(:,4) == noeud,1) > size(Z,1)+1)
            Z = classeArbreRec(Z(Z(:,4) == noeud,1),classe,Z);
        end
        if(Z(Z(:,4) == noeud,2) > size(Z,1)+1)
            Z = classeArbreRec(Z(Z(:,4) == noeud,2),classe,Z);
        end
    else
        if(max(bool2s(Z(:,1) == noeud)) > 0)
            Z(Z(:,1) == noeud,5) = classe;
        else
            Z(Z(:,2) == noeud,5) = classe;
        end
    end
    
endfunction
