function [cm,gn] = nan_confusionmat(g,ghat,order)
//  Confusion matrix for visualization of the performance of an algorithm
//  Calling Sequence
//  CM = nan_confusionmat(G,GHAT)
//  CM = nan_confusionmat(G,GHAT,ORDER)
//  [CM, GORDER] = nan_confusionmat(G, GHAT)
//  Parameters
//  G: actual group labels
//  GHAT: predicted group labels
//  CM : confusion matrix 
//  ORDER: (vector) species the order of rows (and columns) of the confusion matrix 
//   GORDER: order of group labels   for rows and columns of CM. 
//   Description
//    The confusion matrix CM(I,J) represents the count of instances
//    whose actual group labels are group I and whose predicted group labels
//    are group J.
//    
//                   predicted Classes
//                   
//              ___|_____|______| 
//              
//    actual     ' |     |      |
//    
//      Classes    |     |      |
//    
//    If the number of classes is two then the first class is the positive class and the second class is the negative class
//    
//    CM(1,1) is the number of true positive instances
//    
//    CM(2,1) is the number of false negative instances
//    
//    CM(1,2) is the number of false positive instances
//    
//    CM(2,2) is the number of true negative instances
//    
//     nan_confusionmat treats NaNs, empty strings or 'undefined'
//    values in G or GHAT as missing values, and removes those entries
//
//
//   Examples
//      actual=   [0 0 1 1 0 0 0]
//      predicted=[0 1 0 1 0 0 1] 
//      // now we have 3 true positive (0 were classified as 0)
//      // 2 false negatives (0 were incorrectly marked as 1)
//      // 1 false positive (1 were incorrectly marked as 0)
//      // 1 true negatives (1 were correctly classified as 1)
//      [cm,order] = nan_confusionmat(actual,predicted)
//      order  =
//         0
//         1
//      cm =
//         3 2
//         1 1
//
//   See also 
//   nan_grp2idx
//   nan_partest


 [nargout,nargin]=argn(0);
if nargin < 2
    error('At least two inputs are needed.');
elseif nargin==2
  order=[];
end

gClass = typeof(g);
if ~(gClass==typeof(ghat))
    error('g and ghat are not from the same type');
end

if ~or(type(g)==[1 5 8]) & ~type(g)==4   & ~iscell(g)  & ~type(g)==10
    error('g and ghat should be either numerical, logical, a cell or a string matrix.');
end

if type(g)==10
    if length(size(g)) > 2 | length(size(ghat)) > 2
        error('g and ghat must be 2D if they are a string matrix');
    end
    g = g(:);
    ghat = ghat(:);
elseif ~isvector(g) | ~isvector(ghat) 
    error('g and ghat must be a vector!');
else
    g = g(:);
    ghat = ghat(:);
end


if size(g,1) ~= size(ghat,1)
    error('The size of g and ghat should be the same');
end


if ~isempty(order)
    if type(order)==10
        if length(size(order)) > 2
            error('The order variable must be 2d.');
        end
        order = order(:);
    elseif ~isvector(order)
        error( 'the order variable must be a vector.');
    end


        if or(type(g)==[1 5 8])
            if ~or(type(order)==[1 5 8])
                error('order must be the same type as g and ghat.');
            end
            if or(isnan(order))
                error('order should not have any NaN values.');
            end

        elseif type(g)==4
            if type(order)==4
               // ok fine
            elseif or(type(order)==[1 5 8])
                if or(isnan(order))
                    error('order should not have any NaN values.');
                end

                order = (order)~=0;
                
            else
                error('order must the same type as g and ghat');
            end

        elseif iscell(g)
            if ~iscell(order)
                error( 'order must be the same type as g');
            end
            if or((''==(order)))
                error('order should not have any empty strings');
            end
        end

        try
            uorder = unique(order);
        catch 
            error('unique does not work on order');
        end

        if max(size(uorder)) < max(size(order))
            error('order should not have any duplicated values');
        end

        order = order(:);
end


gLen = size(g,1);
[idx,gn,gnn] =nan_grp2idx([g;ghat]);
gidx = idx(1:gLen);
ghatidx = idx(gLen+1:gLen*2);

//ignore NaN values
nanrows = isnan(gidx) | isnan(ghatidx);
if or(nanrows)
    gidx(nanrows) = [];
    ghatidx(nanrows) = [];
end

gLen = size(gidx,1);
gnLen =max(size(gn));

cm = nan_accumarray([gidx,ghatidx], ones(gLen,1),[gnLen, gnLen]);

if ~isempty(order)
    
    [hasAllLabel,map] = nan_ismember(gnn,order);

        if ~and(hasAllLabel)
            error('order must contain all labels which are in g and ghat.');
        end
        orderLen = max(size(order));
        cm2 = zeros(orderLen, orderLen);
        cm2(map,map) = cm(:,:);
        cm = cm2;

    if nargout > 1
        gn = order;

        if (gClass=='string')
            gn = string(gn);
        elseif (gClass=="boolean")
            gn = gn~=0;
        end
    end
else

gn=gnn;

end

endfunction

