function [Nout,Cout] = nan_ecdfhist(F,X,param1,param2,opt_plot)
//   Create histogram from ecdf output.
//   Calling Sequence
//   N = nan_ecdfhist(F,X)
//   N = nan_ecdfhist(F,X,M)
//   N = nan_ecdfhist(F,X,C)
//   [N,Cout] = nan_ecdfhist(...) 
//   nan_ecdfhist(...,'plot') produces a histogram bar plot of   the results.
//   Parameters
//   F: vector of empirical cdf values
//   X: vector of evaluation points
//   M: uses M bins instead of 10
//   C: vector with centers of bins
//   N: heights of   histogram bars for 10 equally spaced bins
//   Cout: position of the bin centers
// Description
//    The bar heights of the histogram are computed   from the increases in the empirical cdf, 
//    and are normalized so the area
//   for each bar represents the probability for the corresponding interval.
//   If F is computed from a censored sample, the total probability may be less
//   than 1.  In contrast, HIST produces bars whose heights are bin counts and
//   whose areas do not represent probabilities.
//
//
//   Examples
//        y = grand(50,1,'exp',10); 
//       [F,X] = nan_ecdf(y);
//       nan_ecdfhist(F,X,'plot');
//       // Superimpose a plot of the known true pdf
//       xx = 0:.1:max(y); yy = exp(-xx/10)/10; plot(xx,yy,'g-');
//
//   See also
//   nan_ecdf
//   nan_histc

[nargout,nargin]=argn(0);


in_par=['F','X','param1','param2','opt_plot'];
in_par_min=2;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;

if nargin>0 & sum(length(F))==1 & typeof(F)=="handle"

    cax = F;
    F = X;
    if nargin > 2
        X = param1;
        if nargin > 3
            param1 = param2;
            param2=[];
        end;
    end
    nargs = nargin - 1;
else
    cax = [];
    nargs = nargin;
end

if nargs<2
    error('Requires both F and X.');
elseif nargs <3 then
     param1=[];
     param2=[];
elseif nargs < 4 then
     param2=[];
end

if type(F)==10 | type(X)==10
    error('F and X must be numeric.')
end
if length(F)~=max(size(F)) | length(X)~=max(size(X))
    error('F and X must be vectors!');
elseif length(F)<2 | length(X)<2
    error( 'F and X must have at least two entries!');
elseif or(diff(X)<0)
    error('X should be non decreasing.');
end

wassurv = (F(1)==1);
if wassurv
    F = 1-F;
end
diffF = diff(F);
if or(diff(F)<0)
        error( 'F must be non-increasing')
end

X = X(2:$);


wng = ''; 
lo = double(min(X(:)));
hi = double(max(X(:)));
useCenters = %t;
if isempty(param1) & isempty(param2)

    binspec = 'nbins';
    nbins = 10;
elseif ~isempty(param1) & isempty(param2)

    if length(param1) == 1 
        nbins = param1;
        binspec = 'nbins';
    else                        
        ctrs = param1(:)'; 
        binspec = 'ctrs';
    end
else
    if type(param1)==10
        select param1
        case 'ctrs'  
            ctrs = param2(:)';
        case 'edges' 
            edges = param2(:)'; 
            useCenters = %f;
        case 'nbins'
            nbins = param2;
        else
            error('You must specify ''nbins'', ''ctrs'', or ''edges''.');
        end
    elseif length(param1) == 1 
        nbins = param1;
        binspec = 'nbins';
    else                        
        ctrs = param1(:)'; 
        binspec = 'ctrs';
    end

end



select binspec
case 'nbins'
    if isempty(X)
        lo = 0;
        hi = 1;
    end
    if lo == hi
        lo = lo - floor(nbins/2) - 0.5;
        hi = hi + ceil(nbins/2) - 0.5;
    end
    binwidth = (hi - lo) ./ nbins;
    edges = lo + binwidth*(0:nbins);
    edges(length(edges)) = hi;
    C = edges(1:$-1) + binwidth/2;
case 'ctrs'
    C = ctrs;
    binwidth = diff(C);
    if or(binwidth<=0)
        error('The vector C of bin centers must be increasing.');
    end
    binwidth = [binwidth binwidth($)];
    edges = [C(1)-binwidth(1)/2 C+binwidth/2];
    if ~isempty(lo)
       edges(1) = min(edges(1),lo);
       edges($) = max(edges($),hi);
    end
case 'edges'
    binwidth = diff(edges);
    if or(binwidth<=0)
        error('The vector E of bin edges must be increasing.');
    end
    if nargout > 2
        C = zeros(binwidth);
        C(1) = edges(1) + binwidth(1)/2;
        for j = 2:length(C)
            C(j) = 2*edges(j) - C(j-1);
        end

        if or(C <= edges(1:$-1)) | abs(C($) - (edges($)-binwidth($)/2)) > 1000*%eps;
            wng = 'Cannot compute centers that are consistent with EDGES.';
            C = edges(1:$-1) + binwidth/2;
        end
    end

    if ~isempty(X) & ((lo < edges(1)) | (edges($) < hi))
        warning('The vector E of bin edges does not span the range of X.');
    end
end


nbins = length(edges) - 1;

if isempty(X)
    binnum = X;
elseif useCenters
    edges = edges + %eps;

    [ignore,binnum] = nan_histc(X,[-%inf edges(2:$-1) %inf]);
else

    [ignore,binnum] = nan_histc(X,edges);

    binnum(binnum==nbins+1) = nbins;
end
        
if or(binnum==0)
    diffF(binnum==0) = [];
    binnum(binnum==0) = [];
end

binnum = binnum(:);
P = nan_accumarray([ones(binnum),binnum],diffF,[1,nbins]);




binwidth = diff(edges);
N = (P ./ binwidth);

if plotting
    if isempty(cax), cax = gca(); end
    if useCenters
        bar(cax,C,N);
        cax.auto_ticks(1)='on'
    else
        bar(cax,edges(1:$-1), N);h=gce();
         cax.auto_ticks(1)='on'

    end
    Nout=[];
else
    Nout = N;
    if nargout>=2
        Cout = C;
        if ~isempty(wng)
            warning(wng);
        end
    end
end

endfunction