function [gidx,gnames,glevels] = nan_grp2idx(s)
//   Create index vector from a grouping variable.
//   Calling Sequence
//   [G,GN] = nan_grp2idx(S)
//   [G,GN,GL] = nan_grp2idx(S)
//   Parameters
//   S: grouping variable, can be a numeric, or logical vector; a cell vector of  strings; or a character matrix with each row representing a group label.
//   G: index vector taking integer values from 1 up to the number K  of distinct groups
//   GN: cell array of strings representing group  labels. GN(G) reproduces S 
//   GL: same as GN, but GL  has the same type as S
// Description
//
//   grp2idx treats NaNs or  empty strings  in S as missing   values and returns NaNs in the corresponding rows of G. 
//   GN and GL don't include entries for missing values.
//
//   See also
//   nan_gscatter
//    nan_grpstats
//   Examples
//     groups={'30s','40s','30s','40s','40s'}'
//     [G,GN]=nan_grp2idx(groups)
//     disp(GN(G))

[nargout,nargin]=argn(0);

if ~isvector(s) 
    error( 'Grouping variable must be a vector or a character array or a cell of strings.');
end
s = s(:);

if or(type(s)==[1 5 8])
    [glevels,dum,gidx] = nan_unique(s,'first');
    

    if ~isempty(glevels) & isnan(glevels($)) 
        glevels = glevels(~isnan(glevels));
        gidx(gidx > length(glevels)) = %nan;
    end
    if nargout > 1
        gnames = string(full(glevels));
    end

elseif type(s)==4
    [glevels,dum,gidx] = nan_unique(bool2s(s),'first');
    
    if ~isempty(glevels) & isnan(glevels($)) 
        glevels = glevels(~isnan(glevels));
        gidx(gidx > length(glevels)) = %nan;
    end
    if nargout > 1
        glevels=(glevels)==1
        gnames = string(glevels);
    end
    
elseif type(s)==10 

    [glevels,ord,gidx] = nan_unique(s,'first');

    [dum,reord] = gsort(ord,'g','i');
    ireord(reord) = 1:length(reord); ireord = ireord(:);
    
    if ~isempty(glevels) & (''==glevels(1)) 
        reord(reord==1) = [];
        ireord = ireord - (ireord > ireord(1));
        ireord(1) = %nan;
    end
    
    gidx = ireord(gidx(:));
    if nargout > 1
        glevels = glevels(reord(:));
        gnames = glevels;
    end
 
 elseif iscell(s)
   
    tmp=string(ones(size(s,1),1));
    for i=1:size(s,1)
       tmp(i)=s.entries(i);
    end
    s=tmp;
    [glevels,ord,gidx] = nan_unique(s,'first');

    [dum,reord] = gsort(ord,'g','i');
    ireord(reord) = 1:length(reord); ireord = ireord(:);
    
    if ~isempty(glevels) & (''==glevels(1)) 
        reord(reord==1) = [];
        ireord = ireord - (ireord > ireord(1));
        ireord(1) = %nan;
    end
    
    gidx = ireord(gidx(:));
    if nargout > 1
        glevels = glevels(reord(:));
        gnames = glevels;
        glevels=cellstr(gnames);
    end
    
else
    error('The input variable must be  numeric, a logical vector, a cell vector of strings, or a 2D character array.');
end
endfunction
