function [varargout] = nan_grpstats(x,groups,whichstats,varargin)
// Summary statistics by group.
// Calling Sequence
// MEANS = nan_grpstats(X,groups)
// nan_grpstats(X,groups,ALPHA) displays a plot of the means versus index with  100(1 - ALPHA)%  confidence intervals around each mean.
// [MEANS,SEM,COUNTS,GNAME] = nan_grpstats(X,groups)
// [A,B,...] = nan_grpstats(X,groups,whichstats)
// [...] = nan_grpstats(X,groups,whichstats,ALPHA)
// Parameters
// X: matrix of observations
// groups: grouping variable defined  as a vector, string array, or list of strings.
//   MEANS: means of each column of X by groups
//   SEM: standard error  of the mean
//   COUNTS:  number of elements in each group
//   GNAME: name of each group
//   ALPHA: specifies the confidence  level as 100(1-ALPHA)% for the 'meanci' and 'predci' options.
//   whichstats:a  list of single function handles or names (strings)
//   the following strings are allowed for whichstats:
//      'mean' :    mean
//      'sem'  :    standard error of the mean
//      'numel':    count, or number of elements
//      'gname':    group name
//      'std'  :    standard deviation
//      'var'  :    variance
//      'min'  :    minimum
//      'max'  :    maximum
//      'range':    maximum - minimum
//      'meanci':   95% confidence interval for the mean
//      'predci':   95% prediction interval for a new observation
//   
//   A,B:  statistics specified by whichstats, The number of outputs [A,B,...] must match the   number function handles and names in whichstats.
//   
//  Description
//   nan_grpstats computes groupwise summary statistics, for data in a matrix or  dataset array.
//   
//   nan_grpstats treats NaNs as missing values, and removes them.
//
// Functions in whichstats must accept a matrix and give out one values for each columns.
//
//   Examples
//      loadmatfile(nan_getpath()+"/demos/data/iris.mat");
//      [m,p,g] = nan_grpstats(meas,species,list('mean','predci','gname'))
//      n = size(m,1)
//      plot2d((1:n)',m)
//      nan_errorbar(ones(1,4) .*. (1:n)',m,p(:,:,2)-m,p(:,:,2)-m)
//      h=gca();
//      h.data_bounds(:,1)=[0.9;3.1];
//      title('95% prediction intervals for mean iris size by species')
//      scf()
//      nan_grpstats(meas,species,0.05)
//
//   See also
//   nan_gscatter
//   nan_grp2idx


[nargout,nargin]=argn(0);
if (nargin<1)
   error('At least one input is needed.')
end
if length(size(x))>2 | ~(or(type(x)==[1,5,8]) | type(x)==4)
    error( 'X must be a numeric or logical vector or matrix');
elseif isvector(x)
    x = x(:);
end
[rows,cols] = size(x);

doplot = %f;
varNames=[];

if nargin==4
    arg4 = varargin(1);
    if nargin==4 & (or(type(arg4)==[1,4,8]) & sum(length(arg4))==1)
        alpha = arg4;
    elseif nargin==4 & or(type(whichstats)==[1,5,8]) & sum(length(whichstats))==1 & type(arg4)==10
        alpha = whichstats;
        whichstats = [];
        doplot = %t;
        varNames=arg4;

    end


elseif nargin==3
    if or(type(whichstats)==[1,5,8]) & sum(length(whichstats))==1
        alpha = whichstats;
        whichstats = [];
        doplot = %t;
    else
        alpha = 0.05;
    end
    
elseif nargin<=2
    alpha = 0.05;   
    whichstats = [];
end

if type(whichstats)~=1 & type(whichstats)~=10 & type(whichstats)~=15 & type(whichstats)~=130 then
  error("whichstats must be a string a function or a list of strings or a list of functions");
end

if alpha<=0 | alpha>=1
    error('ALPHA must be between 0 and 1.');
end


deff("[t]=mean1(x)","t=mean(x,1)");
deff("[t]=sem1(x)","t=nan_std(x,0,1) / sqrt(size(x,1))");
deff("[t]=std1(x)","t=nan_std(x,0,1)");
deff("[t]=var1(x)","t=var(x,0,1)");
deff("[t]=min1(x)","t=empty2NaN(min(x,''r''));");
deff("[t]=max1(x)","t=empty2NaN(max(x,''r''));");
deff("[t]=range1(x)","t=empty2NaN(nan_range(x,1));");
deff("[t]=numel1(x)","t=size(x,1)");


if ~typeof(whichstats)=="list"
      whichstats = list(whichstats);
end

if length(whichstats)==0| length(whichstats(1))==0
    // Default list
    whichstats = list(mean1,sem1,numel1, 'gname');
    if doplot
        minargs = 3;
    else
        minargs = 1;
    end
   for i=max(minargs,nargout)+1:length(whichstats)
       whichstats(i)=null();
    end
else

    for j=1:length(whichstats)
        hfun = whichstats(j);
        if type(hfun)==10
            select(hfun)
              case 'mean' then  hfun = mean1
              case 'sem' then   hfun = sem1
              case 'std' then   hfun = std1
              case 'var' then   hfun = var1
              case 'min' then   hfun = min1
              case 'max' then   hfun = max1
              case 'range' then hfun = range1
              case 'numel' then hfun = numel1
              case 'meanci' then hfun = meanci
              case 'predci' then hfun = predci
              
              case 'gname' then //ok
              else 
                 try
                   hfun=evstr(hfun); 
                 catch
                    error(sprintf("list entry string %s is not known and is also not a known function name!",hfun));
                 end
            end
            whichstats(j) = hfun;
         elseif typeof(hfun)=="function" //|  typeof(hfun)=="fptr"
            select(hfun)
              case mean then  hfun = mean1,
              case nan_sem then   hfun = sem1,
              case nan_std then   hfun = std1,
              case nan_var then   hfun = var1,
              //case min then   hfun = min1,
              //case max then   hfun = max1,
              case range then hfun = range1,
              //case length then hfun = numel1,
              //otherwise, may be a function name or 'gname'
            end
            whichstats(j) = hfun;

        end
    end
    

    if max(1,nargout) < length(whichstats)
        warning(sprintf('nan_grpstats called with %d output arguments to compute %d\n summary statistics.',nargout,length(whichstats)));
    end
end

if (nargin<2) | isempty(groups)
   groups = ones(rows,1);
end
[groups,glabel,groupname,multigroup,ngroups] = nan_mgrp2idx(groups,rows);
if length(groups) ~= rows
    error('Number of groups is not equal to rows of X.');
end

cgroups = list();
for gnum = 1:ngroups
    cgroups(gnum) = find(groups==gnum);
end

nfuns = length(whichstats);

for nfun = 1:nfuns
    hfun = whichstats(nfun);  
    if type(hfun)==10 & (hfun=='gname')
        varargout(nfun) = groupname;
    else

    trymatrix = (cols~=1) & ~or(isnan(x(:)));

    if isempty(cgroups)
        rowidx = [];
    else
        rowidx = cgroups(1);
    end

    if trymatrix
        try
            t = hfun(x(rowidx,:));
            if size(t,2)~=cols
                trymatrix = %f;
            end
        catch
            trymatrix = %f;
        end
    end
    if trymatrix
        nstatvals = size(t,1);
        t1 = matrix(t',[1,cols,nstatvals]);  
        z = repmat(t1,[ngroups,1,1]);          
        tsize = [nstatvals,cols];
    else    

        if size(x,2)>=1
            y = x(rowidx,1);
        else
            y = x(rowidx,[]);
        end
        t = hfun(y(~isnan(y),:));
        nstatvals = size(t,1);
        t1 = matrix(t,[1,1,nstatvals]); 
        z = repmat(t1,[ngroups,cols,1]);   
        tsize = size(t);
        if ngroups>0 & cols>0
            for colnum = 2:cols
                y = x(rowidx,colnum);

                 if length(size(z))==3
                   tmp=  hfun(y(~isnan(y),:));
		    for k=1:size(z(1,colnum,:),3)
		    z(1,colnum,k)=tmp(k,:);
		    end
                 else
                   z(1,colnum) = hfun(y(~isnan(y),:));
                 end
            end
        
      end
    end

    for gnum = 2:ngroups
        idx = cgroups(gnum);

        if trymatrix

            if length(size(z))==3
              tmp= hfun(x(idx,:));
              for k=1:size(z(gnum,:,:),3)
               z(gnum,:,k)=tmp(k,:);
              end
            else
               z(gnum,:) = hfun(x(idx,:));
            end
        else
            for colnum = 1:cols
                y = x(idx,colnum);

               if length(size(z))==3

                 tmp= hfun(y(~isnan(y),:));
                 for k=1:size(z(gnum,colnum,:),3)
                    z(gnum,colnum,k)=tmp(k,:);
                 end
               else
                 z(gnum,colnum) = hfun(y(~isnan(y),:));
               end
            end
        end
    end
    
    if cols==1
        z = matrix(z,ngroups,nstatvals);
    end
    varargout(nfun) = z;

  end;
end

if doplot
   means = varargout(1);
   sems = varargout(2);
   counts = varargout(3);
   p = ones(size(counts,1),size(counts,2))*(1 - alpha/2);
   xd = repmat((1:ngroups)',1,cols);
   plot2d(xd,means);
   nan_errorbar(xd,means,cdft("T",counts-1,p,1-p) .* sems,cdft("T",counts-1,p,1-p) .* sems);
   h=gca();

    if type(glabel)==10
    x_ticks=h.x_ticks;
    x_ticks.locations=(1:ngroups)';
    x_ticks.labels=glabel(:);
    h.x_ticks=x_ticks;
    end
   h.data_bounds(:,1)=[0.5; ngroups+0.5];
   h.data_bounds(:,2)=[min(means-cdft("T",counts-1,p,1-p) .* sems);max(means+cdft("T",counts-1,p,1-p) .* sems)];
   xlabel('Group');
   ylabel('Mean');
   if ~isempty(varNames)
      legend(varNames);
   end
   if (multigroup)
    
      xlabel('');

      ylim=h.data_bounds(:,2);

      for j=1:ngroups
         xstring(j,ylim(1),glabel(j));
      end

   else

   end
   title('Means and Confidence Intervals for Each Group');
   //xgrid(2);
end
endfunction

    function ci = meanci(y) 
    n = size(y,1);
    m = nan_mean(y,1);
    s = nan_std(y,0,1) / sqrt(n);
    d = s * -cdft("T",max(0,n-1),alpha/2,1-(alpha/2) );
    ci = [m-d; m+d];
    endfunction

    // ----------------------------
    function ci = predci(y) 
    n = size(y,1);
    m = nan_mean(y,1);
    s = nan_std(y,0,1) * sqrt(1 + 1/n);
    d = s * -cdft("T",max(0,n-1),alpha/2,1-(alpha/2) );
    ci = [m-d; m+d];
    endfunction

    // ----------------------------
    function m = empty2NaN(m) 
    if size(m,1) == 0
        m = ones(1,size(m,2))*%nan;
    end
    endfunction





