function nan_parallelcoords(X,varargin)
// Parallel coordinates plot for multivariate data.
// Calling Sequence
//  nan_parallelcoords(AX,...) plots into the axes with handle AX.
//  nan_parallelcoords(X)
//  nan_parallelcoords(X,group)
//  nan_parallelcoords(X,group,quant)
//  nan_parallelcoords(X,group,quant,labels)
//  nan_parallelcoords(X,group,quant,labels,stdize)
// Parameters
// AX: handle
// X: Rows of X correspond to   observations, columns to variables
// group: plots the data in different   groups with different colors. group is a string matrix.
// quant: if  not empty the the median and the  quant and (1-quant) quantiles of f(t) at each value of t is plotted
// labels: labels the coordinate tick marks
// stdize = 'off' : do nothing
// stdize ='on' : normalize each column of X (mean = 0, deviation = 1)
// stdize = 'pca' : instead of X   the principal component scores of X is used, in order of decreasing eigenvalue.
// stdize = 'pcastd' : uses the standardized   principal component scores.
// Description
// Parallel coordinates is a common way of visualizing high-dimensional geometry and analyzing multivariate data.
//
//To show a set of points in an n-dimensional space, a backdrop is drawn consisting of n parallel lines, typically vertical and equally spaced. 
//A point in n-dimensional space is represented as a polyline with vertices on the parallel axes; the position of the vertex on the ith axis corresponds to the ith coordinate of the point.
//   
//   nan_parallelcoords treats NaNs in X as missing  values,
//
//   Examples
//
//      // make a grouped plot of the raw data
//     loadmatfile(nan_getpath()+"/demos/data/iris.mat");
//      labels = {'Sepal Length','Sepal Width','Petal Length','Petal Width'};
//      clf();
//      nan_parallelcoords(meas, species, [],labels);
//
//      // plot only the median and quartiles of each group
//      clf();
//      nan_parallelcoords(meas, species, 0.25,labels);
//
//   See also 
//   nan_andrewsplot
//   Bibliography
//   Andrews, D. F. (1972), Plots of high-dimensional data, Biometrics 28, S.125-136 


[nargout,nargin]=argn(0);

if sum(length(X))==1 & typeof(X)=="handle"
    ax = X;

    X = varargin(1);
    varargin(1) = [];
else
    ax = [];
end

if nargin < 1+length(ax)
    error('At least one input argument required.');
end
[n,p] = size(X);

groups=[];
quantil=[];
tcklabs=[];
stdize='off';


if nargin==2 then
groups=varargin(1);
quantil=[];
tcklabs=[];
stdize='off';
elseif nargin==3
groups=varargin(1);
quantil=varargin(2);
tcklabs=[];
stdize='off';
elseif nargin==4
groups=varargin(1);
quantil=varargin(2);
tcklabs=varargin(3);
stdize='off';
elseif nargin==5
groups=varargin(1);
quantil=varargin(2);
tcklabs=varargin(3);
stdize=varargin(4);
end;

if ~isempty(quantil)
    if ~or(type(quantil)==[1 4 8]) | ~sum(length(quantil))==1 | ~(0 < quantil & quantil < 1)
        error( 'quant must be a scalar between 0 and 1.');
    end
    quantil = min(quantil,1-quantil);
end


if isempty(groups)
    gidx = ones(n,1);
    ngroups = 1;
    gnames = [];

else
    [gidx,gnames] = nan_grp2idx(groups);
    ngroups = size(gnames,'*');
    if length(gidx) ~= n
        error( 'the number or rows of groups and X must be the same!');
    end
end

dfltTckLabs = isempty(tcklabs);
if dfltTckLabs
    tcklabs = string(1:p);
elseif type(tcklabs)==10 & size(tcklabs,'*') == p
    // Ok.
else
    error('label must be a vector of strings. Its size must be the same as columns of X');
end


origrows = 1:n;
if isempty(quantil)
    origX = X;
    origQuantiles = [];
else
    origX = [];

    origQuantiles = list();
    for grp = 1:ngroups
        mbrs = find(gidx == grp);
        if length(mbrs) == 1
            q = repmat(X(mbrs,:),3,1); 
        else
            q = nan_prctile(X(mbrs,:), 100*[.50 quantil 1-quantil]);
        end
        origQuantiles(grp) = q;
    end
end


dopca = %f;
rescaled = %t;
if type(stdize)==10
    select convstr(stdize)
    case 'off' then

        rescaled = %f;

    case 'on' then
        if n > 1
            X = (X - repmat(nan_mean(X),n,1))./ repmat(nan_std(X),n,1);
        elseif n == 1
            X(~isnan(X)) = 0; 
        else
            
        end


    case 'pca' then

        nans = find(isnan(gidx) | or(isnan(X),2));
        if ~isempty(nans) 
            X(nans,:) = [];
            gidx(nans) = [];
            origrows(nans) = [];
            n = size(X,1);
        end
        if ~isempty(X)
            [dum,X,variance] = princomp(X); 
        end
        dopca = %t;
     case 'pcastd' then

        nans = find(isnan(gidx) | or(isnan(X),2));
        if ~isempty(nans) 
            X(nans,:) = [];
            gidx(nans) = [];
            origrows(nans) = [];
            n = size(X,1);
        end
        if ~isempty(X)
            [dum,X,variance] = princomp(X); 

                tol = max(n,p)*%eps;
                variance(variance<tol) = 1;
                X = X ./ repmat(sqrt(variance(:)'),n,1);
        end
        dopca = %t;


    else
        error(' stdize must be either ''off'', ''on'', ''pca'', or ''pcastd''.');
    end
else
    error( 'stdize must be a string. ');
end

icoord = 1:p;

cax = gca();
colorlist=["b","r","g","k","y","c","m"];
ncolors = size(colorlist,'*');
hh = list();


lgndh=[];
for grp = 1:ngroups
    colorNr = (pmodulo(grp-1,ncolors)+1);
    mbrs = find(gidx == grp);

    if isempty(mbrs)
        plot(icoord,%nan*ones(size(icoord,1),size(icoord,2)), 'LineStyle','-', 'Color',colorlist(colorNr));


    elseif isempty(quantil)


        plot(icoord,X(mbrs,:)', 'LineStyle','-', 'Color',colorlist(colorNr));
        hh(grp) = gce();
    else
        if length(mbrs) == 1
            per = [.5 .5 .5];
            q = repmat(X(mbrs,:),3,1); 
        else
            per = [.50 quantil 1-quantil];
            q = nan_prctile(X(mbrs,:), 100*per);
        end
        plot(icoord,q(1,:), 'LineStyle','-', 'Color',colorlist(colorNr));
        hh(grp) = gce();
        plot(icoord,q(2:3,:), 'LineStyle',':', 'Color',colorlist(colorNr));
        hh((grp-1)*2 + (2)) = gce();
    end

   
    if ~isempty(groups) & ~isempty(mbrs)
         if isempty(lgndh)
             lgndh=hh(grp).children(1);
         else
              lgndh= [lgndh; hh(grp).children(1)];
         end;
    end
end



    if dopca
        if dfltTckLabs, xlabel('Principal Component'); end
        ylabel('PC Score');
    else
        if dfltTckLabs, xlabel('Coordinate'); end
        ylabel('Coordinate Value');
    end
    x_ticks=cax.x_ticks;
    x_ticks.locations=icoord(:);
    x_ticks.labels=tcklabs(:);
    cax.x_ticks=x_ticks;

if ~isempty(groups)
    tt = 1:size(lgndh,'*');  
    legend(lgndh,gnames(tt,:));
end

endfunction




