function [h,pval, stats] = nan_signrank (x, y, alpha,tail,DIM)
//Wilcoxon signed-rank test
//Calling Sequence
//H=nan_signrank (x, y)
//H=nan_signrank (x, y, alpha)
//H=nan_signrank (x, y, alpha,tail)
//[h,pval, stats] = nan_signrank(..)
//Parameters
// x: input vector
// y: input vector
// alpha: significance   level (default alpha = 0.05)
// tail: 
// H: returns whether the Null-Hypotheses must be rejected. 
// pval:  p-value of the test
// Description
// For two matched-pair sample vectors x and y, perform a
// Wilcoxon signed-rank test of the null hypothesis PROB (x > y) == 1/2.  Under the null, the test statistic @var{z}
// approximately follows a standard normal distribution when n > 5.
//
// @strong{Warning}: This function assumes a normal distribution for z
// and thus is invalid for n < 5.
//
// With the optional argument string tail, the alternative of
// interest can be selected.  If tail is "~=" or "<>", the null is tested against the two-sided alternative
// PROB (x > y) != 1/2.  If alt is ">", the one-sided alternative PROB (x > y) > 1/2 is considered.  Similarly
// for "<", the one-sided alternative PROB (x > y) < 1/2 is considered.  The default is the two-sided case.
// 
// Examples
// Xa=[78,24,64,45,64,52,30,50,64,50,78,22,84,40,90,72];
// Xb=[78,24,62,48,68,56,25,44,56,40,68,36,68,20,58,32];
// 
// 
//  Y1= [81.0 105.4 119.7 109.7  98.3 146.6 142.0 150.7 191.5 145.7  82.3  77.3  78.4 131.3  89.6 119.8 121.4 124.0 140.8 124.8  98.9  89.0  69.1  89.3  104.1  86.9  77.1  78.9 101.8  96.0];
// Y2 = [80.7  82.3  80.4  87.2  84.2 100.4 115.5 112.2 147.7 108.1 103.1 105.1  116.5 139.9 129.6  98.9  61.9  96.2 125.5  75.7  66.4  49.9  96.7  61.9  80.3  67.7  66.7  67.4  91.8  94.1];
// Authors
//  KH Kurt.Hornik@wu-wien.ac.at
//  H. Nahrstaedt - 2012


// Copyright (C) 1995-2011 Kurt Hornik
//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.
[nargout,nargin]=argn(0);

  if (nargin < 2 | nargin > 6)
    error("wrong number of parameters") ;
  end

  if (~ (isvector (x) & isvector (y) & (length (x) == length (y))))
    error ("wilcoxon_test: X and Y must be vectors of the same length");
  end

    if (nargin < 3) | isempty(alpha)
    alpha = .05;
  end

  if (nargin < 4) | isempty(tail)
    tail = '~=';
  end
  if (~ type(tail)==10)
    error ('tail must be a string');
  end
//   if (nargin < 5)
//     method = [];
//   end
// 
//   if ~(method=='exact' | method=='approximate' | isempty(method))
//     error ('method not supported')
//   end	
  if nargin<5,
       DIM = find(size(x)>1,1);
  end;
  if isempty(DIM), DIM=1; end;


  n = length (x);
  x = matrix (x, 1, n);
  y = matrix (y, 1, n);
  d = x - y;
  d(isnan(d)) = [];

  d = d (find (d ~= 0));

  n = length(d);

  if (n==0) then
    pval=1;
    h=%f;
    stats=[];
    return;
  end;

 //  if isempty(method)
    if n<=5
	//method = 'exact';
        error("implementation requires more than 5 different pairs!");
    else
	//method = 'approximate';
    end
 //  end;
    r = nan_ranks (abs (d));
    neg = find(d<0);
    w = sum(r(neg));
    w = min(w, n*(n+1)/2-w);
    stats.signedrank = w;

     z = sum (r (find (d > 0)));
    z = ((z - n * (n + 1) / 4) / sqrt (n * (n + 1) * (2 * n + 1) / 24)); 
    stats.z=z;

    cdf = erfc (z / (-sqrt(2))) / 2;
    if (~ type (tail)==10)
	error("wilcoxon_test: tail must be a string");
      elseif ( (tail== "~=") |   (tail == '!=') | (tail== "<>")) | (tail == 'both'),
	pval = 2 * min (cdf, 1 - cdf);
      elseif ( (tail== ">"))| (tail == 'right'),
	pval = 1 - cdf;
      elseif ( (tail=="<")) | (tail == 'left'),
	pval = cdf;
      else
	error (sprintf("wilcoxon_test: option %s not recognized", tail));
      end


    h = pval <= alpha;	

  if (nargout == 0)
    printf ("  pval: %g\n", pval);
  end

endfunction
