function [h, pval, stats] = nan_signtest (x, y, alpha, tail,DIM)
//Sign test
// Calling Sequence
// H =  nan_signtest (x, y)
// H =  nan_signtest (x, y,alpha)
// H =  nan_signtest (x, y,alpha,tail)
// [H,pval,stats] = nan_signtest(...)
// Parameters
// x: input vector
// y: input vector
// alpha: significance   level (default alpha = 0.05)
// tail: 
// H: returns whether the Null-Hypotheses must be rejected. 
// pval:  p-value of the test
// Description
// 
// For two matched-pair samples x and @y, perform a sign test
// of the null hypothesis PROB (x > y) == PROB (x < y) == 1/2.  Under the null, the test statistic b roughly
// follows a binomial distribution with parameters n = sum (x!= y) and p = 1/2.
//
// With the optional argument tail, the alternative of interest
// can be selected.  If tail is "~=" or "<>", the
// null hypothesis is tested against the two-sided alternative PROB
// (x < y) ~= 1/2.  If tail is ">", the one-sided alternative PROB (x > y) > 1/2 ("x is
// stochastically greater than y") is considered.  Similarly for
// "<", the one-sided alternative PROB (x > y) < 1/2
// ("x is stochastically less than y") is considered.  The default is
// the two-sided case.
//
// Examples
// before=[5 3 4 2 1 6 7 3 2 3 5 1 4 4 3];
// after =[6 2 4 4 3 6 7 5 3 5 5 3 4 5 2];
// nan_signtest(before,after,0.05,"<")
// 
// Authors
// KH Kurt.Hornik@wu-wien.ac.at
// H. Nahrstaedt - 2012

// Copyright (C) 1995-2011 Kurt Hornik
//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.
 [nargout,nargin]=argn(0);


  if ((nargin < 2) | (nargin > 5))
    error("wrong number of parameters") ;
  end

  if (nargin < 3) | isempty(alpha)
    alpha = .05;
  end

  if (nargin < 4) | isempty(tail)
    tail = '~=';
  end
  if (~ type(tail)==10)
    error ('sign_test: tail must be a string');
  end
  if nargin<5,
       DIM = find(size(x)>1,1);
  end;
  if isempty(DIM), DIM=1; end;


  if (~ (isvector (x) & isvector (y) & (length (x) == length (y))))
    error ("sign_test: X and Y must be vectors of the same length");
  end

  n   = length (x);
  x   = matrix (x, 1, n);
  y   = matrix (y, 1, n);
  n   = sum (x ~= y);
  b   = sum (x > y);
  tmp = floor (b);
  cdf = 1 - nan_betainc (1/2, tmp + 1, n - tmp);
  //cdf = 1- cdfbet("XY",tmp+1,n-tmp,1/2,1/2);
  //cdf = binocdf (b, n, 1/2);
  stats.n=n;
  stats.b=b;


   if ( (tail== '~=') |  (tail == '!=') |  (tail == '<>')) | (tail == 'both'),
    pval = 2 * min (cdf, 1 - cdf);
  elseif  (tail ==  '>') | (tail == 'right'),
    pval = 1 - cdf;
  elseif  (tail == '<') | (tail == 'left'),
    pval = cdf;
  else
    error (sprintf("sign_test: option %s not recognized", alt));
  end

  h = pval <= alpha;	

  if (nargout == 0)
    printf ("  pval: %g\n", pval);
  end

endfunction
