function [h, pval, ci, stats] = nan_ttest (x, m, alpha, tail, DIM)
//  (paired) t-test
// Calling Sequence
// H = nan_ttest(x,m)
//	tests Null-hypothesis that mean of x is m. 		
// H = nan_ttest(x,y)
// 	size of x and size of y must match, it is tested whether the 
//	difference x-y is significantly different to m=0; 
// H = nan_ttest(x,y,alpha)
// H = nan_ttest(x,y,alpha,tail)
// H = nan_ttest(x,y,alpha,tail,DIM)
// [H,pval,ci,stats] = nan_ttest(...)
// Parameters
// x: input vector
// m: mean value for testing
// y: input vector
// alpha: significance   level (default alpha = 0.05)
// tail:
// DIM:
// H: returns whether the Null-Hypotheses must be rejected. 
// pval:  p-value of the test
// ci: confidence internvals (depending on alpha)
// Description
//     For a sample X from a normal distribution with unknown mean and
//     variance, perform a t-test of the null hypothesis `mean (X) == M'.
//     Under the null, the test statistic T follows a Student
//     distribution with `DF = length (X) - 1' degrees of freedom.
//
//     TTEST treads NaNs as "Missing values" and ignores these. 
//
//     H=%T indicates a rejection of the Null-hypothesis at a significance 
//     level of alpha (default alpha = 0.05).	 
// 
//     With the optional argument string TAIL, the alternative of interest
//     can be selected.  If TAIL is '!=' or '<>' or 'both', the null is tested
//     against the two-sided Alternative `mean (X) ~= mean (Y)'.  If TAIL
//     is '>' or 'right', the one-sided Alternative `mean (X) > mean (Y)' is used.
//     Similarly for '<' or 'left', the one-sided Alternative `mean (X) < mean
//     (Y)' is used.  The default is the two-sided case.
// 
//     H returns whether the Null-Hypotheses must be rejected. 
//     The p-value of the test is returned in PVAL. 
// 
//     TTEST works on the first non-singleton dimension or on DIM. 
//
//     If no output argument is given, the p-value of the test is
//     displayed.
// Example
// before=[223, 259, 248, 220, 287, 191, 229, 270, 245, 201];
// after =[220, 244, 243, 211, 299, 170, 210, 276, 252, 189];
// [H,pval]=nan_ttest(before,mean(after)); // Null-hypothesis will not be rejected
// // the treatment does not have any effect or the number is to small
// [H,pval]=nan_ttest(before,after,0.05,'<'); // Null-hypothesis will not be rejected
// // the treatment does not reduce the value.
// Authors
// Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007    Kurt Hornik
// Copyright (C) 2010 by Alois Schloegl a.schloegl@ieee.org
// H. Nahrstaedt - 2010



//       $Id$
//       Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007
//               Kurt Hornik
//       Copyright (C) 2010 by Alois Schloegl <a.schloegl@ieee.org>	
//       This function is part of the NaN-toolbox
//       http://biosig-consulting.com/matlab/NaN/

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 [nargout,nargin]=argn(0);
  if ((nargin < 2) | (nargin > 5) | nargout > 4)
        error("wrong number of parameters") ;
  end

  if (nargin < 3) | isempty(alpha)
    alpha = .05;
  end

  if (nargin < 4) | isempty(tail)
    tail = '~=';
  end
  if (~ type(tail)==10)
    error ('nan_ttest: tail must be a string');
  end

  if nargin<5,
       DIM = find(size(x)>1,1);
  end;
  if isempty(DIM), DIM=1; end;

  szx = size(x); 
  szm = size(m);	
  szx(DIM) = 1;	  
  szm(DIM) = 1;
  if size(m,DIM)==1
  	;
  elseif size(x,DIM) == size(m,DIM)
  	x = x-m;
  	m = mtlb_zeros(szm);
  else
    error ('nan_ttest: dimension of X and Y do not fit');
  end 	  
  xmean=nan_mean(x,DIM);
  [S, N] = sumskipnan(x, DIM);
  stats.df = N - 1;
  stats.sd = nan_std (x,[],DIM);
  stats.tstat = sqrt (N) .* (S./N - m) ./ stats.sd;
  //cdf = tcdf (stats.tstat, stats.df);
  cdf = cdft("PQ",stats.tstat,stats.df);

  if ( (tail == '~=') |  (tail == '!=') |  (tail ==  '<>')) | (tail == 'both'),
    pval = 2 * min (cdf, 1 - cdf);
    crit = distfun_tinv((1 - alpha / 2), stats.df) .* (stats.sd ./sqrt(N));
    ci = cat(DIM, xmean - crit, xmean  + crit);
  elseif  (tail == '>') | (tail == 'right'),
    crit = distfun_tinv((1 - alpha ), stats.df) .* (stats.sd ./sqrt(N));
    ci = cat(DIM, xmean - crit, %inf);
    pval = 1 - cdf;
  elseif  (tail == '<') | (tail == 'left'),
    crit = distfun_tinv((1 - alpha ), stats.df) .* (stats.sd ./sqrt(N));
    ci = cat(DIM, -%inf, xmean  + crit);
    pval = cdf;
  else
    error (sprintf('nan_ttest: option %s not recognized', tail));
  end

  h = pval < alpha;	
  if (nargout == 0)
    printf('  pval: %g\n', pval);
  end
endfunction
