function [h, pval, ci, stats, df] = nan_ttest2 (x, y, alpha, tail, vartype, DIM)
//  (unpaired) t-test
// Calling Sequence
// H = nan_ttest2(x,y)
// H = nan_ttest2(x,y,alpha)
// H = nan_ttest2(x,y,alpha,tail)
// H = nan_ttest2(x,y,alpha,tail,vartype)
// H = nan_ttest2(x,y,alpha,tail,vartype,DIM)
// [H,pval] = nan_ttest2(...)
// [H ,pval,ci,stats] = nan_ttest2(...)
// Parameters
// x: input vector
// y: input vector
// alpha: significance   level (default alpha = 0.05)
// tail: 
// vartype: supports 'equal' (default value) and 'unequal'. 
// DIM: 
// H: returns whether the Null-Hypotheses must be rejected. 
// pval:  p-value of the test
// ci: confidence internvals (depending on alpha)
// Description
//     For two samples x and y from normal distributions with unknown
//     means and unknown equal variances, perform a two-sample t-test of
//     the null hypothesis of equal means.  Under the null, the test
//     statistic T follows a Student distribution with DF degrees of
//     freedom.
//
//     nan_ttest2 treads NaNs as "Missing values" and ignores these. 
//
//     H=%t indicates a rejection of the Null-hypothesis at a significance 
//     level of alpha (default alpha = 0.05).	 
// 
//     With the optional argument string TAIL, the Alternative of interest
//     can be selected.  If TAIL is '~=' or '<>' or 'both', the null is tested
//     against the two-sided Alternative `mean (X) ~= mean (Y)'.  If TAIL
//     is '>' or 'right', the one-sided Alternative `mean (X) > mean (Y)' is used.
//     Similarly for '<' or 'left', the one-sided Alternative `mean (X) < mean
//     (Y)' is used.  The default is the two-sided case.
// 
//     vartype supports 'equal' (default value) and 'unequal'. 
//

// 
//     nan_ttest2 works on the first non-singleton dimension or on DIM. 
//

// Authors
//       Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007       Kurt Hornik
//       Copyright (C) 2010 by Alois Schloegl a.schloegl@ieee.org
//       H. Nahrstaedt - 2010


//       $Id$
//       Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007
//               Kurt Hornik
//       Copyright (C) 2010 by Alois Schloegl <a.schloegl@ieee.org>	
//       This function is part of the NaN-toolbox
//       http://biosig-consulting.com/matlab/NaN/

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 [nargout,nargin]=argn(0);
  if ((nargin < 2) | (nargin > 6) | nargout > 4)
      error("wrong usage");
  end

  if (nargin < 3) | isempty(alpha)
    alpha = .05;
  end

  if (nargin < 4) | isempty(tail)
    tail = '~=';
  end
  if (~ type(tail)==10)
    error ('ttest2: tail must be a string');
  end
  if (nargin < 5) | isempty(vartype)
    vartype = 'equal';
  end
  if ~(vartype=='equal' | vartype=='unequal')
    error ('test: vartype not supported')
  end	
  if nargin<6,
       DIM = find(size(x)>1,1);
  end;
  if isempty(DIM), DIM=1; end;

  szx = size(x); 
  szy = size(y);	
  szy(DIM) = 1;	  
  szx(DIM) = 1;
  
  if (or(szx-szy))
    error ('ttest2: dimension of X and Y do not fit');
  end

  [SX, NX] = sumskipnan(x, DIM);
  [SY, NY] = sumskipnan(y, DIM);
  MX = SX ./ NX;
  MY = SY ./ NY;

  if vartype=='equal' then
    stats.df = NX + NY - 2;
    sPooled = sqrt(((NX-1) .* nan_var(x,[],DIM) + (NY-1) .* nan_var(y,[],DIM)) ./ stats.df);
    se = sPooled .* sqrt(1 ./NX + 1 ./NY);


    if or(size(x)==0) | or(size(y)==0)
	  v = %nan;
    else	
	  v = nan_sumsq(x-repmat(MX,size(x)./size(MX)),DIM) + nan_sumsq(y-repmat(MY,size(y)./size(MY)),DIM);
    end; 	
    stats.sd    = sqrt(v./stats.df);
    stats.tstat = (MX - MY) .* sqrt ((NX .* NY .* stats.df) ./ (v .* (NX + NY)));
  else
    s2xbar = nan_var(x,[],DIM) ./ NX;
    s2ybar = nan_var(y,[],DIM) ./ NY;
    stats.df = (s2xbar + s2ybar) .^2 ./ (s2xbar.^2 ./ (NX-1) + s2ybar.^2 ./ (NY-1));
    stats.sd= sqrt(cat(DIM, nan_var(x,[],DIM), nan_var(y,[],DIM)))
    se = sqrt(s2xbar + s2ybar);
    stats.tstat = (MX-MY) ./ se;

  end
  //cdf         = tcdf (stats.tstat, stats.df);
  cdf = cdft("PQ",stats.tstat,stats.df);

  if ( (tail== '~=') |  (tail == '!=') |  (tail == '<>')) | (tail == 'both'),
    pval = 2 * min (cdf, 1 - cdf);
     crit = distfun_tinv((1 - alpha / 2), stats.df) .* se;
    ci = cat(DIM,(nan_mean(x,DIM)-nan_mean(y,DIM))-crit,(nan_mean(x,DIM)-nan_mean(y,DIM))+crit);
  elseif  (tail ==  '>') | (tail == 'right'),
    pval = 1 - cdf;
     crit = distfun_tinv((1 - alpha ), stats.df) .* se;
    ci = cat(DIM,(nan_mean(x,DIM)-nan_mean(y,DIM))-crit,%inf);
  elseif  (tail == '<') | (tail == 'left'),
    pval = cdf;
    crit = distfun_tinv((1 - alpha), stats.df) .* se;
    ci = cat(DIM,-%inf,(nan_mean(x,DIM)-nan_mean(y,DIM))+crit);
  else
    error (sprintf('ttest2: option %s not recognized', tail));
  end

  h = pval < alpha;	

  if (nargout == 0)
    printf('  pval: %g\n', pval);
  end
endfunction
