function [h,pval, stats] = nan_welchtest (x, y,  alpha, tail,DIM)
//Welch two-sample t test
// Calling Sequence
// H = nan_welchtest (x, y)
// H = nan_welchtest (x, y,  alpha)
// H = nan_welchtest (x, y,  alpha, tail)
// [H,pval, stats] = nan_welchtest ( ... )
// Parameters
// x: input vector
//y: input vector
// tail: 
// H: returns whether the Null-Hypotheses must be rejected. 
// pval:  p-value of the tested
// Description
//
// For two samples x and y from normal distributions with
// unknown means and unknown and not necessarily equal variances,
// perform a Welch test of the null hypothesis of equal means.
// Under the null, the test statistic t approximately follows a
// Student distribution with @var{df} degrees of freedom.
//
// With the optional argument string tail, the alternative of
// interest can be selected.  If tail is "~=" or
// "<>", the null is tested against the two-sided alternative
// mean (x) ~= m.  If tail is ">", the
// one-sided alternative mean(x) > @var{m} is considered.  Similarly for
// "<", the one-sided alternative mean(x) < m is
// considered.  The default is the two-sided case.
//
// Authors
//  Kurt Hornik Kurt.Hornik@wu-wien.ac.at
//  H. Nahrstaedt - 2012


// Copyright (C) 1995-2011 Kurt Hornik
//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.


 [nargout,nargin]=argn(0);

  if ((nargin < 2) | (nargin > 5))
    error("wrong number of parameters") ;
  end


 if (nargin < 3) | isempty(alpha)
    alpha = .05;
  end

  if (nargin < 4) | isempty(tail)
    tail = '~=';
  end
  if (~ type(tail)==10)
    error ('ttest2: tail must be a string');
  end

  if nargin<5,
       DIM = find(size(x)>1,1);
  end;
  if isempty(DIM), DIM=1; end;

  if (~ (isvector (x) & isvector (y)))
    error ("welch_test: both X and Y must be vectors");
  end

  n_x  = length (x);
  n_y  = length (y);
  mu_x = sum (x) / n_x;
  mu_y = sum (y) / n_y;
  v_x  = nan_sumsq (x - mu_x) / (n_x * (n_x - 1));
  v_y  = nan_sumsq (y - mu_y) / (n_y * (n_y - 1));
  c    = v_x / (v_x + v_y);
  df   = 1 / (c^2 / (n_x - 1) + (1 - c)^2 / (n_y - 1));
  t    = (mu_x - mu_y) / sqrt (v_x + v_y);
  //cdf  = tcdf (t, df);
  cdf = cdft("PQ",t,df);

  stats.t=t;
  stats.df=df;

  if ( (tail== '~=') |  (tail == '!=') |  (tail == '<>')) | (tail == 'both'),
    pval = 2 * min (cdf, 1 - cdf);
  elseif  (tail ==  '>') | (tail == 'right'),
    pval = 1 - cdf;
  elseif  (tail == '<') | (tail == 'left'),
    pval = cdf;
  else
    error (sprintf("welch_test: option %s not recognized", alt));
  end

  h = pval < alpha;

  if (nargout == 0)
    printf ("  pval: %g\n", pval);
  end

endfunction
