function [h,pval, z, ci] = nan_ztest (x, m, sigma,alpha,tail,DIM)
//Test for mean of a normal sample with known variance
//Calling Sequence
//H = nan_ztest (x, m, sigma,alpha,tail)
//H = nan_ztest (m1, sigma1, n1,m2, tail,alpha)
//[H,pval, z, ci] = nan_ztest (...)
// Parameters
// x: input vector
//  m: mean value for testing
//  sigma: known variance of x
//  alpha: significance   level (default alpha = 0.05)
//  tail:
//  DIM
// H: returns whether the Null-Hypotheses must be rejected. 
// pval:  p-value of the test
// ci: confidence internvals (depending on alpha)
// Description
// Perform a Z-test of the null hypothesis mean (x) == m for a sample x from a normal distribution with unknown
// mean and known variance sigma.  Under the null, the test statistic
// z follows a standard normal distribution.
//
// With the optional argument string tail, the alternative of
// interest can be selected.  If tail is "~=" or "<>", the null is tested against the two-sided alternative
// mean (x) != m.  If tail is ">", the one-sided alternative mean (x) > m is considered.
// Similarly for "<", the one-sided alternative mean (x) < m is considered.  The default is the two-sided
// case.
//
//
// Authors
// KH Kurt.Hornik@wu-wien.ac.at
// H. Nahrstaedt - 2012

// Copyright (C) 1995-2011 Kurt Hornik
//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.

 [nargout,nargin]=argn(0);

  if ((nargin < 3) | (nargin > 6))
    error("wrong number of parameters") ;
  end

  if (~ isvector (x)) then
    if ((nargin < 4) | (nargin > 6))
     error("wrong number of parameters") ;
    end
    if nargin >= 6 then
      tail=DIM;
    else
      tail = "~=";
    end;

    if nargin >= 5 then
      alpha=tail;
    else
      alpha=0.05;
    end;

    m2=alpha;
    n=sigma;
    sigma=m;
    m=x;


    s = sigma ./ sqrt(n);
    z = (m2-m)/s;
    cdf = erfc (z / (-sqrt(2))) / 2;

  else
    if (~ length (m)==1)
      error ("z_test: M must be a scalar");
    end
    if (~ (length (sigma)==1 & (sigma > 0)))
      error ("z_test: sigma must be a positive scalar");
    end

    n = max(size (x));
    s = sigma ./ sqrt(n);
    z = (sum (x) / n - m)/s;
    cdf = erfc (z / (-sqrt(2))) / 2;



    if (nargin == 3)
      tail = "~=";
    end
  end;

  if (~ type (tail)==10)
    error ("z_test: ALT must be a string");
  elseif ( (tail== "~=") |  (tail == '!=') |  (tail== "<>")) | (tail == 'both'), 
    pval = 2 * min (cdf, 1 - cdf);
    crit = sqrt(2)*erfinv(2*(1 - alpha/2) - 1) .* s;
    ci = cat(1, m-crit, m+crit);
  elseif ( (tail== ">")) | (tail == 'right'),
    pval = 1 - cdf;
    crit = sqrt(2)*erfinv(2*(1 - alpha) - 1) .* s;
    ci = cat(1, m-crit, %inf);
  elseif ( (alt== "<")) | (tail == 'left'),
    pval = cdf;
    crit = sqrt(2)*erfinv(2*(1 - alpha) - 1) .* s;
    ci = cat(1, -%inf, m+crit);
  else
    error (sprintf("z_test: option %s not recognized\n", tail));
  end

  h = pval < alpha;

  if (nargout == 0)
    s = strcat (["Z-test of mean(x) == %g against mean(x) %s %g,\n",                "with known var(x) == %g:\n",                "  pval = %g\n"]);
    printf (s, m, tail, m, v, pval);
  end

endfunction
